@@ -95,7 +95,7 @@ import net.percederberg.mibble.value.ValueReference;
  * identifier is encountered. 
  *
  * @author   Per Cederberg, <per at percederberg dot net>
- * @version  2.0
+ * @version  2.2
  * @since    2.0
  */
 class MibAnalyzer extends Asn1Analyzer {
@@ -125,6 +125,11 @@ class MibAnalyzer extends Asn1Analyzer {
      */
     private ArrayList contextStack = new ArrayList();
                                                                                 
+    /**
+     * The implicit tags flag.
+     */
+    private boolean implicitTags = true;
+
     /**
      * Creates a new MIB file analyzer.
      * 
@@ -300,6 +305,30 @@ class MibAnalyzer extends Asn1Analyzer {
         return node;
     }
 
+    /**
+     * Sets the implicit tags flag.
+     * 
+     * @param node           the node being exited
+     * 
+     * @return null to remove the node from the parse tree
+     * 
+     * @throws ParseException if the node analysis discovered errors
+     */
+    protected Node exitTagDefault(Production node)
+        throws ParseException {
+
+        Node  child;
+
+        child = getChildAt(node, 0);
+        if (child.getId() == Asn1Constants.EXPLICIT) {
+            implicitTags = false;
+        } else {
+            implicitTags = true;
+        }
+        return null;
+    }
+
+
     /**
      * Adds all imported MIB files to the MIB context. Also removes
      * this node from the parse tree. 
@@ -430,8 +459,7 @@ class MibAnalyzer extends Asn1Analyzer {
 
         // Create type symbol
         type = (MibType) getValue(getChildAt(node, 2), 0);
-        symbol = new MibTypeSymbol(getLocation(node), name, type);
-        mib.addSymbol(symbol);
+        symbol = new MibTypeSymbol(getLocation(node), mib, name, type);
 
         return null;
     }
@@ -518,7 +546,7 @@ class MibAnalyzer extends Asn1Analyzer {
      * @return the node to add to the parse tree
      */
     protected Node exitNullType(Production node) {
-        node.addValue(NullType.TYPE);
+        node.addValue(new NullType());
         return node;
     }
 
@@ -530,7 +558,7 @@ class MibAnalyzer extends Asn1Analyzer {
      * @return the node to add to the parse tree
      */
     protected Node exitBooleanType(Production node) {
-        node.addValue(BooleanType.TYPE);
+        node.addValue(new BooleanType());
         return node;
     }
 
@@ -542,7 +570,7 @@ class MibAnalyzer extends Asn1Analyzer {
      * @return the node to add to the parse tree
      */
     protected Node exitRealType(Production node) {
-        node.addValue(RealType.TYPE);
+        node.addValue(new RealType());
         return node;
     }
 
@@ -581,7 +609,7 @@ class MibAnalyzer extends Asn1Analyzer {
      * @return the node to add to the parse tree
      */
     protected Node exitObjectIdentifierType(Production node) {
-        node.addValue(ObjectIdentifierType.TYPE);
+        node.addValue(new ObjectIdentifierType());
         return node;
     }
 
@@ -696,7 +724,7 @@ class MibAnalyzer extends Asn1Analyzer {
         // TODO: implement set type support
         log.addError(getLocation(node), 
                      "SET type currently unsupported");
-        node.addValue(NullType.TYPE);
+        node.addValue(new NullType());
         return node;
     }
 
@@ -712,7 +740,7 @@ class MibAnalyzer extends Asn1Analyzer {
         // TODO: implement set of type support
         log.addError(getLocation(node), 
                      "SET OF type currently unsupported");
-        node.addValue(NullType.TYPE);
+        node.addValue(new NullType());
         return node;
     }
 
@@ -740,7 +768,7 @@ class MibAnalyzer extends Asn1Analyzer {
         // TODO: implement enumerated type support
         log.addError(getLocation(node), 
                      "ENUMERATED type currently unsupported");
-        node.addValue(NullType.TYPE);
+        node.addValue(new NullType());
         return node;
     }
 
@@ -756,7 +784,7 @@ class MibAnalyzer extends Asn1Analyzer {
         // TODO: implement selection type support
         log.addError(getLocation(node), 
                      "selection type currently unsupported");
-        node.addValue(NullType.TYPE);
+        node.addValue(new NullType());
         return node;
     }
 
@@ -772,16 +800,95 @@ class MibAnalyzer extends Asn1Analyzer {
     protected Node exitTaggedType(Production node)
         throws ParseException {
 
-        MibType  type;
-        Node     child;
-        
-        // TODO: handle the type tag?
+        MibType     type;
+        MibTypeTag  tag;
+        boolean     implicit = implicitTags;
+        Node        child;
+
+        child = getChildAt(node, 0);
+        tag = (MibTypeTag) getValue(child, 0);
+        child = getChildAt(node, 1);
+        if (child.getId() == Asn1Constants.EXPLICIT_OR_IMPLICIT_TAG) {
+            implicit = ((Boolean) getValue(child, 0)).booleanValue();
+        }
         child = getChildAt(node, node.getChildCount() - 1);
         type = (MibType) getValue(child, 0);
+        type.setTag(implicit, tag);
         node.addValue(type);
         return node;
     }
 
+    /**
+     * Called when exiting a parse tree node.
+     * 
+     * @param node           the node being exited
+     * 
+     * @return the node to add to the parse tree, or
+     *         null if no parse tree should be created
+     */
+    protected Node exitTag(Production node) {
+        ArrayList  values = getChildValues(node);
+        int        category = MibTypeTag.CONTEXT_SPECIFIC_CATEGORY;
+        int        value;
+        
+        if (values.size() == 1) {
+            value = ((Number) values.get(0)).intValue();
+        } else {
+            category = ((Integer) values.get(0)).intValue();
+            value = ((Number) values.get(1)).intValue();
+        }
+        node.addValue(new MibTypeTag(category, value));
+        return node;
+    }
+
+    /**
+     * Adds the type tag category value as a node value.
+     * 
+     * @param node           the node being exited
+     * 
+     * @return the node to add to the parse tree
+     * 
+     * @throws ParseException if the node analysis discovered errors
+     */
+    protected Node exitClass(Production node) throws ParseException {
+        Node  child = getChildAt(node, 0);
+        int   category;
+        
+        if (child.getId() == Asn1Constants.UNIVERSAL) {
+            category = MibTypeTag.UNIVERSAL_CATEGORY;
+        } else if (child.getId() == Asn1Constants.APPLICATION) {
+            category = MibTypeTag.APPLICATION_CATEGORY;
+        } else if (child.getId() == Asn1Constants.PRIVATE) {
+            category = MibTypeTag.PRIVATE_CATEGORY;
+        } else {
+            category = MibTypeTag.CONTEXT_SPECIFIC_CATEGORY;
+        }
+        node.addValue(new Integer(category));
+        return node;
+    }
+
+    /**
+     * Adds the implicit boolean flag as a node value.
+     * 
+     * @param node           the node being exited
+     * 
+     * @return the node to add to the parse tree
+     * 
+     * @throws ParseException if the node analysis discovered errors
+     */
+    protected Node exitExplicitOrImplicitTag(Production node)
+        throws ParseException {
+
+        Node  child = getChildAt(node, 0);
+        
+        if (child.getId() == Asn1Constants.EXPLICIT) {
+            node.addValue(Boolean.FALSE);
+        } else {
+            node.addValue(Boolean.TRUE);
+        }
+        return node;
+    }
+
     /**
      * Adds a null type as a node value. This method also prints an
      * error about this construct being unsupported.
@@ -794,7 +901,7 @@ class MibAnalyzer extends Asn1Analyzer {
         // TODO: implement any type support
         log.addError(getLocation(node), 
                      "ANY type currently unsupported");
-        node.addValue(NullType.TYPE);
+        node.addValue(new NullType());
         return node;
     }
 
@@ -901,7 +1008,8 @@ class MibAnalyzer extends Asn1Analyzer {
         
         name = getStringValue(getChildAt(node, 0), 0);
         value = (MibValue) getValue(getChildAt(node, 2), 0);
-        symbol = new MibValueSymbol(getLocation(node), 
+        symbol = new MibValueSymbol(getLocation(node),
+                                    null, 
                                     name, 
                                     null, 
                                     value);
@@ -1208,8 +1316,7 @@ class MibAnalyzer extends Asn1Analyzer {
         // Create value symbol
         type = (MibType) getValue(getChildAt(node, 1), 0);
         value = (MibValue) getValue(getChildAt(node, 3), 0);
-        symbol = new MibValueSymbol(getLocation(node), name, type, value);
-        mib.addSymbol(symbol);
+        symbol = new MibValueSymbol(getLocation(node), mib, name, type, value);
         return null;
     }
 
