@@ -56,7 +56,7 @@ import java.util.ArrayList;
  * concurrently in multiple threads. 
  *
  * @author   Per Cederberg, <per at percederberg dot net>
- * @version  2.0
+ * @version  2.2
  * @since    2.0
  */
 public class MibLoader {
@@ -177,11 +177,12 @@ public class MibLoader {
 
     /**
      * Returns a previously loaded MIB file. If the MIB file hasn't
-     * been loaded, null will be returned.
+     * been loaded, null will be returned. The MIB is identified by
+     * it's MIB name (i.e. the module name).
      * 
-     * @param name           the MIB name
+     * @param name           the MIB (module) name
      * 
-     * @return the MIB container if found, or
+     * @return the MIB module if found, or
      *         null otherwise
      */
     public Mib getMib(String name) {
@@ -195,6 +196,22 @@ public class MibLoader {
         }
         return null;
     }
+    
+    /**
+     * Returns all previously loaded MIB files. If no MIB files have
+     * been loaded an empty array will be returned.
+     * 
+     * @return an array with all loaded MIB files
+     * 
+     * @since 2.2
+     */
+    public Mib[] getAllMibs() {
+        Mib[]  res;
+        
+        res = new Mib[mibs.size()];
+        mibs.toArray(res);
+        return res;
+    }
 
     /**
      * Searches for a MIB in the search path. The name specified
@@ -231,7 +248,9 @@ public class MibLoader {
 
     /**
      * Loads a MIB file. The file is searched for in the MIB search
-     * path.
+     * path. The MIB is identified by it's MIB name (i.e. the module 
+     * name). This method will also load all imported MIB:s if not 
+     * previously loaded by this loader.
      * 
      * @param name           the MIB name (filename without extension)
      * 
@@ -256,7 +275,8 @@ public class MibLoader {
     }
     
     /**
-     * Loads a MIB file.
+     * Loads a MIB file. This method will also load all imported 
+     * MIB:s if not previously loaded by this loader.
      * 
      * @param file           the MIB file
      * 
@@ -274,7 +294,8 @@ public class MibLoader {
     }
 
     /**
-     * Loads a MIB.
+     * Loads a MIB. This method will also load all imported MIB:s if
+     * not previously loaded by this loader.
      * 
      * @param src            the MIB source
      * 
@@ -300,6 +321,52 @@ public class MibLoader {
         return (Mib) mibs.get(position);
     }
 
+    /**
+     * Schedules the loading of a MIB file. The file is added to the
+     * queue of MIB files to be loaded, unless it is already loaded 
+     * or in the queue. The file is searched for in the MIB search
+     * path.
+     * 
+     * @param name           the MIB name (filename without extension)
+     * 
+     * @throws FileNotFoundException if the MIB file couldn't be 
+     *             found in the MIB search path
+     */
+    void scheduleLoad(String name) throws FileNotFoundException {
+        MibSource  src;
+        
+        src = locate(name);
+        if (src == null) {
+            throw new FileNotFoundException("couldn't locate MIB: '" + 
+                                            name + "'");
+        }
+        scheduleLoad(src);
+    }
+
+    /**
+     * Schedules the loading of a MIB file. The file is added to the
+     * queue of MIB files to be loaded, unless it is already loaded 
+     * or in the queue.
+     * 
+     * @param file           the MIB file
+     */
+    void scheduleLoad(File file) {
+        scheduleLoad(new MibSource(file));
+    }
+
+    /**
+     * Schedules the loading of a MIB. The MIB source is added to the
+     * queue of MIB:s to be loaded, unless it is already loaded or in
+     * the queue.
+     * 
+     * @param src            the MIB source
+     */
+    void scheduleLoad(MibSource src) {
+        if (!mibs.contains(src.getFile()) && !queue.contains(src)) {
+            queue.add(src);
+        }
+    }
+
     /**
      * Loads all MIB files in the loader queue. New entries may be
      * added to the queue while loading a MIB, as a result of 
@@ -354,52 +421,6 @@ public class MibLoader {
         return log;
     }
 
-    /**
-     * Schedules the loading of a MIB file. The file is added to the
-     * queue of MIB files to be loaded, unless it is already loaded 
-     * or in the queue. The file is searched for in the MIB search
-     * path.
-     * 
-     * @param name           the MIB name (filename without extension)
-     * 
-     * @throws FileNotFoundException if the MIB file couldn't be 
-     *             found in the MIB search path
-     */
-    public void scheduleLoad(String name) throws FileNotFoundException {
-        MibSource  src;
-        
-        src = locate(name);
-        if (src == null) {
-            throw new FileNotFoundException("couldn't locate MIB: '" + 
-                                            name + "'");
-        }
-        scheduleLoad(src);
-    }
-
-    /**
-     * Schedules the loading of a MIB file. The file is added to the
-     * queue of MIB files to be loaded, unless it is already loaded 
-     * or in the queue.
-     * 
-     * @param file           the MIB file
-     */
-    public void scheduleLoad(File file) {
-        scheduleLoad(new MibSource(file));
-    }
-
-    /**
-     * Schedules the loading of a MIB. The MIB source is added to the
-     * queue of MIB:s to be loaded, unless it is already loaded or in
-     * the queue.
-     * 
-     * @param src            the MIB source
-     */
-    private void scheduleLoad(MibSource src) {
-        if (!mibs.contains(src.getFile()) && !queue.contains(src)) {
-            queue.add(src);
-        }
-    }
-
 
     /**
      * A MIB input source. This class encapsulates the two different 
