@@ -45,10 +45,10 @@ import net.percederberg.mibble.MibValueSymbol;
  * identifier values in a tree hierarchy.
  *
  * @author   Per Cederberg, <per at percederberg dot net>
- * @version  2.0
+ * @version  2.2
  * @since    2.0
  */
-public class ObjectIdentifierValue implements MibValue {
+public class ObjectIdentifierValue extends MibValue {
 
     /**
      * The component parent.
@@ -82,6 +82,7 @@ public class ObjectIdentifierValue implements MibValue {
      * @param value          the component value
      */
     public ObjectIdentifierValue(String name, int value) {
+        super("OBJECT IDENTIFIER");
         this.parent = null;
         this.name = name;
         this.value = value;
@@ -98,6 +99,7 @@ public class ObjectIdentifierValue implements MibValue {
                                  String name, 
                                  int value) {
 
+        super("OBJECT IDENTIFIER");
         this.parent = parent;
         this.name = name;
         this.value = value;
@@ -115,6 +117,7 @@ public class ObjectIdentifierValue implements MibValue {
                                  String name, 
                                  int value) {
 
+        super("OBJECT IDENTIFIER");
         this.parent = parent;
         this.name = name;
         this.value = value;
@@ -122,10 +125,13 @@ public class ObjectIdentifierValue implements MibValue {
 
     /**
      * Initializes the MIB value. This will remove all levels of
-     * indirection present, such as references to other values, and 
-     * returns the basic value. No value information is lost by this 
-     * operation. This method may modify this object as a 
-     * side-effect, and will be called by the MIB loader.
+     * indirection present, such as references to other values. No 
+     * value information is lost by this operation. This method may 
+     * modify this object as a side-effect, and will return the basic 
+     * value.<p>
+     * 
+     * <strong>NOTE:</strong> This is an internal method that should
+     * only be called by the MIB loader.
      * 
      * @param log            the MIB loader log
      * 
@@ -155,6 +161,23 @@ public class ObjectIdentifierValue implements MibValue {
         return this;
     }
 
+    /**
+     * Creates a value reference to this value. The value reference 
+     * is normally an identical value. Only certain values support 
+     * being referenced, and the default implementation of this 
+     * method throws an exception.<p> 
+     * 
+     * <strong>NOTE:</strong> This is an internal method that should
+     * only be called by the MIB loader.
+     * 
+     * @return the MIB value reference
+     * 
+     * @since 2.2
+     */
+    public MibValue createReference() {
+        return this;
+    }
+
     /**
      * Checks if this object equals another object. This method will
      * compare the string representations for equality.
